# -*- coding: utf-8 -*-

import tempfile
import subprocess
import traceback
import sys
import os
from os.path import isfile
from math import pi
import cairo
from .utils import get_path


DEFAULT_UNIT = 'm'


def dots(size, unit, dpi):
    if size.startswith('pt'):
        size = float(size[2:], ) / 72.0
    else:
        size = float(size)
        if unit == 'm':
            size = size / 25.4  # convert mm to inch
        elif unit != 'i':
            raise ValueError(
                'invalid unit: %s' % repr(unit)
            )
    return dpi * size


def draw_label_file(label_file, cr, dpi, get_image_file):
    # cr.set_font_size(dpi * 6.0 / 72)
    if label_file is None:
        return
    unit = DEFAULT_UNIT
    for line in label_file.splitlines():
        if line.startswith('m'):
            unit = line[2:]  # m or i
            if unit == 'i':
                # print('unit is inch')
                pass
            elif unit == 'm':
                # print('unit is mm')
                pass
            else:
                print('unsupported line:', line)
        elif line.startswith('J'):
            if len(line) != 1:
                print('unsupported line:', line)
        elif line.startswith('S'):
            # size TBD
            pass
        elif line.startswith('O'):
            # TBD
            pass
        elif line.startswith('A'):
            # TBD
            pass
        elif line.startswith('T'):
            # print("preview", line)
            control, text = line.split(';', 1)
            control = control[2:]
            pos_x, pos_y, rotate, font, size = control.split(',')
            # print("text", pos_x, pos_y, rotate, font, size, text)

            size = dots(size, unit, dpi)
            cr.set_font_size(size)
            if font == '3':
                cr.select_font_face(
                    "Swiss 721",
                    cairo.FONT_SLANT_NORMAL,
                    cairo.FONT_WEIGHT_NORMAL,
                )
            elif font == '596':
                cr.select_font_face(
                    "Monospace 821",
                    cairo.FONT_SLANT_NORMAL,
                    cairo.FONT_WEIGHT_NORMAL,
                )
            elif font == '5':
                cr.select_font_face(
                    "Swiss 721",
                    cairo.FONT_SLANT_NORMAL,
                    cairo.FONT_WEIGHT_BOLD,
                )
            # elif font == '10':
            #     cr.select_font_face(
            #         "Arial",
            #         cairo.FONT_SLANT_NORMAL,
            #         cairo.FONT_WEIGHT_NORMAL,
            #         )
            # elif font == '11':
            #     cr.select_font_face(
            #         "Arial",
            #         cairo.FONT_SLANT_NORMAL,
            #         cairo.FONT_WEIGHT_BOLD,
            #         )
            # elif font == '12':
            #     cr.select_font_face(
            #         "Trebuchet MS",
            #         cairo.FONT_SLANT_NORMAL,
            #         cairo.FONT_WEIGHT_NORMAL,
            #         )
            # elif font == '13':
            #     cr.select_font_face(
            #         "Trebuchet MS",
            #         cairo.FONT_SLANT_NORMAL,
            #         cairo.FONT_WEIGHT_BOLD,
            #         )
            else:
                print('unsupported font number:', font)
            cr.set_source_rgb(0, 0, 0)
            if rotate == '0':
                # textents = cr.text_extents(text)
                # print('text:', text, 'textents:', textents)
                # print(
                #     'pos_x_dots:', dots(pos_x, unit, dpi),
                #     'pos_y_dots:', dots(pos_y, unit, dpi)
                #     )
                cr.move_to(
                    dots(pos_x, unit, dpi),  # - textents[0],
                    dots(pos_y, unit, dpi),
                )
                cr.show_text(text)
            elif rotate == '90':
                # textents = cr.text_extents(text)
                cr.move_to(
                    dots(pos_x, unit, dpi),
                    dots(pos_y, unit, dpi),  # + textents[0]
                )
                cr.save()
                cr.rotate(1.5 * pi)
                cr.show_text(text)
                cr.restore()
            elif rotate == '270' or rotate == '-90':
                # textents = cr.text_extents(text)
                cr.move_to(
                    dots(pos_x, unit, dpi),
                    dots(pos_y, unit, dpi),  # - textents[0]
                )
                cr.save()
                cr.rotate(0.5 * pi)
                cr.show_text(text)
                cr.restore()
            elif rotate == '180':
                # textents = cr.text_extents(text)
                cr.move_to(
                    dots(pos_x, unit, dpi),  # + textents[0],
                    dots(pos_y, unit, dpi),
                )
                cr.save()
                cr.rotate(pi)
                cr.show_text(text)
                cr.restore()
            else:
                print('unsupported rotate:', rotate)
        elif line.startswith('G'):
            # print("preview", line)
            # G 0.05,0.64,0;R:1.9,0.33,0.0066666
            control, text = line.split(';', 1)
            control = control[2:]
            pos_x, pos_y, rotate = control.split(',')
            function, function_params = text.split(':', 1)
            if function == 'R':
                width, height, thickness = function_params.split(',', 2)
                cr.set_line_width(dots(thickness, unit, dpi))
                cr.rectangle(
                    dots(pos_x, unit, dpi),
                    dots(pos_y, unit, dpi),
                    dots(width, unit, dpi),
                    dots(height, unit, dpi),
                )
                cr.set_source_rgb(0, 0, 0)
                cr.stroke()
            else:
                print('unsupported line:', line)
        elif line.startswith('B'):
            # print("preview", line)
            # B 1.39,0.08,0,QRCODE+EL{error level}+MODEL2,.0133333;{qrcode}
            control, text = line.split(';', 1)
            control = control[2:]
            pos_x, pos_y, rotate, barcode, size = control.split(',', 4)
            barcode_opts = barcode.split('+')
            if barcode_opts[0] == 'QRCODE':
                qr_model = 1
                error_correction = 'L'
                for opt in barcode_opts:
                    if opt == 'QRCODE':
                        pass
                    elif opt == 'MODEL1':
                        qr_model = 1
                    elif opt == 'MODEL2':
                        qr_model = 2
                    elif opt == 'ELL':
                        error_correction = 'L'
                    elif opt == 'ELM':
                        error_correction = 'M'
                    elif opt == 'ELQ':
                        error_correction = 'Q'
                    elif opt == 'ELH':
                        error_correction = 'H'
                    else:
                        print('unsupported qrcode option:', opt)
                box_size = int(0.5 + dots(size, unit, dpi))
                if qr_model != 2:
                    print('unsupported QR Code model:', qr_model)
                try:
                    with tempfile.NamedTemporaryFile(
                        suffix='.png',
                        prefix='qr',
                        delete=False,
                    ) as qr_file:
                        qr_file_name = qr_file.name
                    args = [
                        'qrencode',
                        '-o', qr_file_name,
                        '-l', error_correction,
                        '-m', '0',
                        '-s', str(box_size),
                        text
                    ]
                    # print("args:", args)
                    subprocess.check_call(args)
                    surface = cairo.ImageSurface.create_from_png(
                        qr_file_name
                    )
                    # cr.set_antialias(cairo.ANTIALIAS_NONE)
                    cr.set_source_surface(
                        surface,
                        dots(pos_x, unit, dpi),
                        dots(pos_y, unit, dpi)
                    )
                    cr.paint()
                except subprocess.CalledProcessError:
                    traceback.print_exc(file=sys.stdout)
                finally:
                    try:
                        if isfile(qr_file_name):
                            os.remove(qr_file_name)
                            # print('removed temporary file:', qr_file_name)
                    except OSError:
                        print('cannot remove temporary file:', qr_file_name)
                        traceback.print_exc(file=sys.stdout)
            else:
                print('unsupported line:', line)
        elif line.startswith('I'):
            # I [:name;]x,y,r[,mx,my,GOODBADn][,a];name
            # implement:
            # I x,y,r;name
            control, image_name = line.split(';', 1)
            control = control[2:]
            pos_x, pos_y, rotate = control.split(',', 2)
            # print('image_name:', repr(image_name))
            image_file_name = get_image_file(image_name)
            # print('image_file_name:', repr(image_file_name))
            surface = cairo.ImageSurface.create_from_png(
                get_path(image_file_name)
            )
            cr.set_source_surface(
                surface,
                dots(pos_x, unit, dpi),
                dots(pos_y, unit, dpi)
            )
            cr.paint()
        else:
            print('unsupported line:', line)
