# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function


def shift_and_format_items(
        items,
        x_shift=None,
        y_shift=None,
        label_format_params=None
):
    retval = ''
    for item in items:
        retval += shift(
            item.format(
                **label_format_params
            ) if label_format_params is not None else item,
            x_shift,
            y_shift
        ) + '\r\n'
    return retval


def shift(line, x_shift, y_shift=None):
    """
    item is a JScript line
    """
    if line.startswith('T'):
        # print("preview", line)
        if x_shift is None or float(x_shift) == 0.0:
            if y_shift is None or float(y_shift) == 0.0:
                return line
        control, text = line.split(';', 1)
        control = control[2:]
        pos_x, pos_y, rotate, font, size = control.split(',')
        if x_shift is not None and float(x_shift) != 0.0:
            pos_x = str(float(pos_x) + float(x_shift))
        if y_shift is not None and float(y_shift) != 0.0:
            pos_y = str(float(pos_y) + float(y_shift))
        control = ','.join((pos_x, pos_y, rotate, font, size))
        control = 'T ' + control
        return ';'.join((control, text))
    elif line.startswith('G'):
        # print("preview", line)
        # G 0.05,0.64,0;R:1.9,0.33,0.0066666
        if x_shift is None or float(x_shift) == 0.0:
            if y_shift is None or float(y_shift) == 0.0:
                return line
        control, text = line.split(';', 1)
        control = control[2:]
        pos_x, pos_y, rotate = control.split(',')
        if x_shift is not None and float(x_shift) != 0.0:
            pos_x = str(float(pos_x) + float(x_shift))
        if y_shift is not None and float(y_shift) != 0.0:
            pos_y = str(float(pos_y) + float(y_shift))
        control = ','.join((pos_x, pos_y, rotate))
        control = 'G ' + control
        return ';'.join((control, text))
    elif line.startswith('B'):
        # print("preview", line)
        # B 1.39,0.08,0,QRCODE+EL{error level}+MODEL2,.0133333;{qrcode}
        if x_shift is None or float(x_shift) == 0.0:
            if y_shift is None or float(y_shift) == 0.0:
                return line
        control, text = line.split(';', 1)
        control = control[2:]
        pos_x, pos_y, rotate, barcode, size = control.split(',', 4)
        if x_shift is not None and float(x_shift) != 0.0:
            pos_x = str(float(pos_x) + float(x_shift))
        if y_shift is not None and float(y_shift) != 0.0:
            pos_y = str(float(pos_y) + float(y_shift))
        control = ','.join((pos_x, pos_y, rotate, barcode, size))
        control = 'B ' + control
        return ';'.join((control, text))
    elif line.startswith('I'):
        # print("preview", line)
        # B 1.39,0.08,0,QRCODE+EL{error level}+MODEL2,.0133333;{qrcode}
        if x_shift is None or float(x_shift) == 0.0:
            if y_shift is None or float(y_shift) == 0.0:
                return line
        control, text = line.split(';', 1)
        control = control[2:]
        pos_x, pos_y, rotate = control.split(',', 2)
        if x_shift is not None and float(x_shift) != 0.0:
            pos_x = str(float(pos_x) + float(x_shift))
        if y_shift is not None and float(y_shift) != 0.0:
            pos_y = str(float(pos_y) + float(y_shift))
        control = ','.join((pos_x, pos_y, rotate))
        control = 'I ' + control
        return ';'.join((control, text))
    else:
        print('unsupported line:', line)
