# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import print_function
from yaml import load
from yaml import dump
from sys import stdout
from glob import glob
from os.path import join
from os.path import dirname
from os.path import basename
from .utils import get_path

reference_config_file = get_path('reference.yaml')
reference_dir = join(dirname(__file__), 'reference')


class MultipleFiles(Exception):
    pass


class MissingFile(Exception):
    pass


class ReferenceConfig(object):
    def __init__(self, config_file_path):
        with open(config_file_path, 'r') as stream:
            yaml_data = load(stream)
        # print('yaml_data:', repr(yaml_data))
        # yaml.dump(yaml_data, sys.stdout, default_flow_style=False)
        if 'part numbers' in yaml_data:
            self.part_numbers = yaml_data['part numbers']
            # print('label types:', repr(self.label_types))
        else:
            raise ValueError('"part numbers" is not found in config file')
        if 'spds' in yaml_data:
            self.spds = yaml_data['spds']
            # print('label types:', repr(self.label_types))
        else:
            raise ValueError('"spds" is not found in config file')

    def get_file(self, file_name):
        files = glob(
            join(
                reference_dir,
                file_name + '*'
            )
        )
        if len(files) > 1:
            raise MultipleFiles(str(files))
        if len(files) == 0:
            raise MissingFile(file_name)
        # print('get_file:', files[0])
        return files[0]

    def get_item(self, spd, document_title):
        return self.spds[spd]['items'][document_title]

    def check_references(self):
        with open(reference_config_file, 'r') as stream:
            reference = load(stream)
            dump(reference, stdout, default_flow_style=False)
        referred_files = []
        missing_files = set()
        all_label_part_numbers = set()
        for spd_name, spd_data in self.spds.items():
            spd_files = glob(
                join(
                    reference_dir,
                    spd_name + '*'
                )
            )
            referred_files.extend(spd_files)
            if len(spd_files) == 0:
                missing_files.add(spd_name)
            if 'items' in spd_data:
                for title, item_data in spd_data['items'].items():
                    if 'qr code standard' in item_data:
                        qr_code_standard = item_data['qr code standard']
                        qr_code_standard_files = glob(
                            join(
                                reference_dir,
                                qr_code_standard + '*'
                            )
                        )
                        if len(qr_code_standard_files) == 0:
                            missing_files.append(qr_code_standard)
                        else:
                            referred_files.extend(qr_code_standard_files)
                    if 'part number' in item_data:
                        label_part_number = item_data['part number']
                        all_label_part_numbers.add(label_part_number)
                        label_files = glob(
                            join(
                                reference_dir,
                                label_part_number + '*'
                            )
                        )
                        if len(label_files) == 0:
                            missing_files.add(label_part_number)
                            # print(title, label_part_number)
                        else:
                            referred_files.extend(label_files)
        all_files = glob(join(reference_dir, '*'))
        # print('all files:', all_files)
        extra_files = []
        for file_name in all_files:
            if file_name not in referred_files:
                extra_files.append(basename(file_name))
        if missing_files:
            print('missing reference files:', missing_files)
        if extra_files:
            print("extra reference files:", extra_files)
        if not missing_files and not extra_files:
            print("reference files checked OK.")
        missing_spds = []
        referenced_spds = []
        extra_spds = []
        for part_number, part_number_data in self.part_numbers.items():
            if 'spd' in part_number_data:
                spd = part_number_data['spd']
                if spd not in self.spds:
                    missing_spds.append(spd)
                else:
                    referenced_spds.append(spd)
        for spd_name, spd_data in self.spds.items():
            if spd_name not in referenced_spds:
                extra_spds.append(spd_name)
        if missing_spds:
            print('missing SPDs:', missing_spds)
        if extra_spds:
            print('extra SPDs:', extra_spds)
        if not missing_spds and not extra_spds:
            print("SPDs checked OK.")
        print("all_label_part_numbers:", all_label_part_numbers)


def check_references():
    reference_config = ReferenceConfig(reference_config_file)
    reference_config.check_references()
